/* This file is a part of the SimpleScalar tool suite written by
 * Todd M. Austin, University of Wisconsin - Madison, Computer Sciences
 * Department as a part of the Multiscalar Research Project.
 *  
 * The SimpleScalar x86 port was written by Steve Bennett.
 * 
 * The tool suite is currently maintained by Doug Burger.
 * 
 * Copyright (C) 1994, 1995, 1996 by Todd M. Austin
 *
 * This source file is distributed "as is" in the hope that it will be
 * useful.  The tool set comes with no warranty, and no author or
 * distributor accepts any responsibility for the consequences of its
 * use. 
 * 
 * Everyone is granted permission to copy, modify and redistribute
 * this tool set under the following conditions:
 * 
 *    This source code is distributed for non-commercial use only. 
 *    Please contact the maintainer for restrictions applying to 
 *    commercial use.
 *
 *    Permission is granted to anyone to make or distribute copies
 *    of this source code, either as received or modified, in any
 *    medium, provided that all copyright notices, permission and
 *    nonwarranty notices are preserved, and that the distributor
 *    grants the recipient permission for further redistribution as
 *    permitted by this document.
 *
 *    Permission is granted to distribute this file in compiled
 *    or executable form under the same conditions that apply for
 *    source code, provided that either:
 *
 *    A. it is accompanied by the corresponding machine-readable
 *       source code,
 *    B. it is accompanied by a written offer, with no time limit,
 *       to give anyone a machine-readable copy of the corresponding
 *       source code in return for reimbursement of the cost of
 *       distribution.  This written offer must permit verbatim
 *       duplication by anyone, or
 *    C. it is distributed by someone who received only the
 *       executable form, and is accompanied by a copy of the
 *       written offer of source code that they received concurrently.
 *
 * In other words, you are welcome to use, share and improve this
 * source file.  You are forbidden to forbid anyone else to use, share
 * and improve what you give them.
 *
 * INTERNET: dburger@cs.wisc.edu
 * US Mail:  1210 W. Dayton Street, Madison, WI 53706
 */

#ifndef HELPER_H
#define HELPER_H

#include "types.h"
#include "translate.h"

/* rotate instructions */
extern inline Bit8u  rr_b(Bit8u op,  Bit8u bop, int ror, cpu_t *state);
extern inline Bit16u rr_w(Bit16u op, Bit8u bop, int ror, cpu_t *state);
extern inline Bit32u rr_d(Bit32u op, Bit8u bop, int ror, cpu_t *state);
extern inline Bit8u  rl_b(Bit8u op,  Bit8u bop, int rol, cpu_t *state);
extern inline Bit16u rl_w(Bit16u op, Bit8u bop, int rol, cpu_t *state);
extern inline Bit32u rl_d(Bit32u op, Bit8u bop, int rol, cpu_t *state);

/* shift instructions */
extern inline Bit8u  shl_b(Bit8u op1, Bit8u bop2, cpu_t *state);
extern inline Bit8u  shr_b(Bit8u op1, Bit8u bop2, cpu_t *state);
extern inline Bit8u  sar_b(Bit8u op1, Bit8u bop2, cpu_t *state);
extern inline Bit16u shl_w(Bit16u op1,Bit8u bop2, cpu_t *state);
extern inline Bit16u shr_w(Bit16u op1,Bit8u bop2, cpu_t *state);
extern inline Bit16u sar_w(Bit16u op1,Bit8u bop2, cpu_t *state);
extern inline Bit32u shl_d(Bit32u op1,Bit8u bop2, cpu_t *state);
extern inline Bit32u shr_d(Bit32u op1,Bit8u bop2, cpu_t *state);
extern inline Bit32u sar_d(Bit32u op1,Bit8u bop2, cpu_t *state);
extern inline Bit32u bsf(Bit32u dop, cpu_t *state);
extern inline Bit32u bsr(Bit32u dop, cpu_t *state);
extern Bit16u shrd_w(Bit16u wop1, Bit16u wop2, Bit8u bop, cpu_t *state);
extern Bit32u shrd_d(Bit32u dop1, Bit32u dop2, Bit8u bop, cpu_t *state);
extern Bit16u shld_w(Bit16u wop1, Bit16u wop2, Bit8u bop, cpu_t *state);
extern Bit32u shld_d(Bit32u dop1, Bit32u dop2, Bit8u bop, cpu_t *state);

/* flags */
extern inline void byteFlagsSZP(Bit8u res,  cpu_t *state);
extern inline void wordFlagsSZP(Bit16u res, cpu_t *state);
extern inline void doubFlagsSZP(Bit32u res, cpu_t *state);
extern inline void byteFlagsAddOSZAP(Bit8u bop1,  Bit8u bop2,  Bit8u bres,  cpu_t *state);
extern inline void wordFlagsAddOSZAP(Bit16u wop1, Bit16u wop2, Bit16u wres, cpu_t *state);
extern inline void doubFlagsAddOSZAP(Bit32u dop1, Bit32u dop2, Bit32u dres, cpu_t *state);
extern inline void byteFlagsSubOSZAP(Bit8u bop1,  Bit8u bop2,  Bit8u bres,  cpu_t *state);
extern inline void wordFlagsSubOSZAP(Bit16u wop1, Bit16u wop2, Bit16u wres, cpu_t *state);
extern inline void doubFlagsSubOSZAP(Bit32u dop1, Bit32u dop2, Bit32u dres, cpu_t *state);
extern inline void logicalByteFlags(Bit8u bres,  cpu_t *state);
extern inline void logicalWordFlags(Bit16u wres, cpu_t *state);
extern inline void logicalDoubFlags(Bit32u dres, cpu_t *state);
extern inline void standardAddByteFlags(Bit8u bop1,  Bit8u bop2,  Bit8u bres,  cpu_t *state);
extern inline void standardAddWordFlags(Bit16u wop1, Bit16u wop2, Bit16u wres, cpu_t *state);
extern inline void standardAddDoubFlags(Bit32u dop1, Bit32u dop2, Bit32u dres, cpu_t *state);
extern inline void standardSubByteFlags(Bit8u bop1,  Bit8u bop2,  Bit8u bres,  cpu_t *state);
extern inline void standardSubWordFlags(Bit16u wop1, Bit16u wop2, Bit16u wres, cpu_t *state);
extern inline void standardSubDoubFlags(Bit32u dop1, Bit32u dop2, Bit32u dres, cpu_t *state);

/* string instructions */
extern inline void stos(int size, iInst_t inst, cpu_t *state);
extern inline void lods(int size, iInst_t inst, cpu_t *state);
extern inline void cmps(int size, iInst_t inst, cpu_t *state);
extern inline void scas(int size, iInst_t inst, cpu_t *state);
extern inline void movs(int size, iInst_t inst, cpu_t *state);



extern Float64 round(Float64 fop, int round_mode);

extern Float64 check_add64(Float64 op1, Float64 op2, Float64 res);


#endif
