/* This file is a part of the SimpleScalar tool suite written by
 * Todd M. Austin, University of Wisconsin - Madison, Computer Sciences
 * Department as a part of the Multiscalar Research Project.
 *  
 * The SimpleScalar x86 port was written by Steve Bennett.
 * 
 * The tool suite is currently maintained by Doug Burger.
 * 
 * Copyright (C) 1994, 1995, 1996 by Todd M. Austin
 *
 * This source file is distributed "as is" in the hope that it will be
 * useful.  The tool set comes with no warranty, and no author or
 * distributor accepts any responsibility for the consequences of its
 * use. 
 * 
 * Everyone is granted permission to copy, modify and redistribute
 * this tool set under the following conditions:
 * 
 *    This source code is distributed for non-commercial use only. 
 *    Please contact the maintainer for restrictions applying to 
 *    commercial use.
 *
 *    Permission is granted to anyone to make or distribute copies
 *    of this source code, either as received or modified, in any
 *    medium, provided that all copyright notices, permission and
 *    nonwarranty notices are preserved, and that the distributor
 *    grants the recipient permission for further redistribution as
 *    permitted by this document.
 *
 *    Permission is granted to distribute this file in compiled
 *    or executable form under the same conditions that apply for
 *    source code, provided that either:
 *
 *    A. it is accompanied by the corresponding machine-readable
 *       source code,
 *    B. it is accompanied by a written offer, with no time limit,
 *       to give anyone a machine-readable copy of the corresponding
 *       source code in return for reimbursement of the cost of
 *       distribution.  This written offer must permit verbatim
 *       duplication by anyone, or
 *    C. it is distributed by someone who received only the
 *       executable form, and is accompanied by a copy of the
 *       written offer of source code that they received concurrently.
 *
 * In other words, you are welcome to use, share and improve this
 * source file.  You are forbidden to forbid anyone else to use, share
 * and improve what you give them.
 *
 * INTERNET: dburger@cs.wisc.edu
 * US Mail:  1210 W. Dayton Street, Madison, WI 53706
 */

#ifndef SYSCALL_H
#define SYSCALL_H

#include "ix86_state.h"

/*****************************************************
 SYSTEM CALL NUMBERS
 ******************************************************/
/* these constants can be found in .../linux/arch/i386/kernel/entry.S */
#define	LINUX_SYS_setup		0
#define	LINUX_SYS_exit		1
#define	LINUX_SYS_fork		2
#define	LINUX_SYS_read		3
#define	LINUX_SYS_write		4
#define	LINUX_SYS_open		5
#define	LINUX_SYS_close		6
#define	LINUX_SYS_waitpid	7
#define	LINUX_SYS_creat		8
#define	LINUX_SYS_link		9
#define	LINUX_SYS_unlink	10
#define	LINUX_SYS_execve	11
#define	LINUX_SYS_chdir		12
#define	LINUX_SYS_time		13
#define	LINUX_SYS_mknod		14
#define	LINUX_SYS_chmod		15
#define	LINUX_SYS_chown		16
#define	LINUX_SYS_break		17
#define	LINUX_SYS_stat		18
#define	LINUX_SYS_lseek		19
#define	LINUX_SYS_getpid	20
#define	LINUX_SYS_mount		21
#define	LINUX_SYS_umount	22
#define	LINUX_SYS_setuid	23
#define	LINUX_SYS_getuid	24
#define	LINUX_SYS_stime		25
#define	LINUX_SYS_ptrace	26
#define	LINUX_SYS_alarm		27
#define	LINUX_SYS_fstat		28
#define	LINUX_SYS_pause		29
#define	LINUX_SYS_utime		30
#define	LINUX_SYS_stty		31
#define	LINUX_SYS_gtty		32
#define	LINUX_SYS_access	33
#define	LINUX_SYS_nice		34
#define	LINUX_SYS_ftime		35
#define	LINUX_SYS_sync		36
#define	LINUX_SYS_kill		37
#define	LINUX_SYS_rename	38
#define	LINUX_SYS_mkdir		39
#define	LINUX_SYS_rmdir		40
#define	LINUX_SYS_dup		41
#define	LINUX_SYS_pipe		42
#define	LINUX_SYS_times		43
#define	LINUX_SYS_prof		44
#define	LINUX_SYS_brk		45
#define	LINUX_SYS_setgid	46
#define	LINUX_SYS_getgid	47
#define	LINUX_SYS_signal	48
#define	LINUX_SYS_geteuid	49
#define	LINUX_SYS_getegid	50
#define	LINUX_SYS_acct		51
#define	LINUX_SYS_phys		52
#define	LINUX_SYS_lock		53
#define	LINUX_SYS_ioctl		54
#define	LINUX_SYS_fcntl		55
#define	LINUX_SYS_mpx		56
#define	LINUX_SYS_setpgid	57
#define	LINUX_SYS_ulimit	58
#define LINUX_SYS_olduname	59
#define	LINUX_SYS_umask		60
#define	LINUX_SYS_chroot	61
#define LINUX_SYS_ustat		62
#define	LINUX_SYS_dup2		63
#define	LINUX_SYS_getppid	64
#define	LINUX_SYS_getpgrp	65
#define	LINUX_SYS_setsid	66
#define	LINUX_SYS_sigaction	67
#define	LINUX_SYS_sgetmask	68
#define	LINUX_SYS_sigsetmask	69
#define	LINUX_SYS_setreuid	70
#define	LINUX_SYS_setregid	71
#define	LINUX_SYS_sigsuspend	72
#define	LINUX_SYS_sigpending	73
#define	LINUX_SYS_sethostname	74
#define	LINUX_SYS_setrlimit	75
#define	LINUX_SYS_getrlimit	76
#define	LINUX_SYS_getrusage	77
#define	LINUX_SYS_gettimeofday	78
#define	LINUX_SYS_settimeofday	79
#define	LINUX_SYS_getgroups	80
#define	LINUX_SYS_setgroups	81
#define	LINUX_SYS_select	82
#define	LINUX_SYS_symlink	83
#define	LINUX_SYS_lstat		84
#define	LINUX_SYS_readlink	85
#define	LINUX_SYS_uselib	86
#define	LINUX_SYS_swapon	87
#define	LINUX_SYS_reboot	88
#define	LINUX_SYS_readdir	89
#define	LINUX_SYS_mmap		90
#define	LINUX_SYS_munmap	91
#define	LINUX_SYS_truncate	92
#define	LINUX_SYS_ftruncate	93
#define	LINUX_SYS_fchmod	94
#define	LINUX_SYS_fchown	95
#define	LINUX_SYS_getpriority	96
#define	LINUX_SYS_setpriority	97
#define	LINUX_SYS_profil	98
#define LINUX_SYS_statfs      	99
#define LINUX_SYS_fstatfs     	100
#define	LINUX_SYS_ioperm	101
#define	LINUX_SYS_socketcall	102
#define	LINUX_SYS_syslog	103
#define	LINUX_SYS_getitimer	104
#define	LINUX_SYS_setitimer	105
#define	LINUX_SYS_newstat	106
#define	LINUX_SYS_newlstat	107
#define	LINUX_SYS_newfstat	108
#define	LINUX_SYS_uname		109
#define	LINUX_SYS_iopl		110
#define	LINUX_SYS_vhangup	111
#define	LINUX_SYS_idle		112
#define	LINUX_SYS_vm86		113
#define	LINUX_SYS_wait4		114
#define	LINUX_SYS_swapoff	115
#define	LINUX_SYS_sysinfo	116
#define	LINUX_SYS_ipc		117
#define	LINUX_SYS_fsync		118
#define LINUX_SYS_sigreturn	119
#define LINUX_SYS_clone         120
#define LINUX_SYS_setdomainname	121
#define	LINUX_SYS_newuname	122
#define	LINUX_SYS_modify_ldt	123
#define	LINUX_SYS_adjtimex	124
#define	LINUX_SYS_mprotect	125
#define	LINUX_SYS_sigprocmask	126
#define	LINUX_SYS_create_module	127
#define	LINUX_SYS_init_module	128
#define	LINUX_SYS_delete_module		129
#define	LINUX_SYS_get_kernel_syms	130
#define LINUX_SYS_quotactl    	131
#define	LINUX_SYS_getpgid	132
#define	LINUX_SYS_fchdir	133
#define	LINUX_SYS_bdflush	134
#define LINUX_SYS_sysfs         135
#define LINUX_SYS_personality   136
#define LINUX_SYS_unused        137
#define LINUX_SYS_setfsuid      138
#define LINUX_SYS_setfsgid      139
#define LINUX_SYS_bllseek       140
/* end system call definitions */



/*****************************************************
 IOCTL commands
 ******************************************************/
/* from <linux/termios.h> */
#define LINUX_TCGETS          0x5401
#define LINUX_TCSETS          0x5402
#define LINUX_TCSETSW         0x5403
#define LINUX_TCSETSF         0x5404
#define LINUX_TCGETA          0x5405
#define LINUX_TCSETA          0x5406
#define LINUX_TCSETAW         0x5407
#define LINUX_TCSETAF         0x5408
#define LINUX_TCSBRK          0x5409
#define LINUX_TCXONC          0x540A
#define LINUX_TCFLSH          0x540B
#define LINUX_TIOCEXCL        0x540C
#define LINUX_TIOCNXCL        0x540D
#define LINUX_TIOCSCTTY       0x540E
#define LINUX_TIOCGPGRP       0x540F
#define LINUX_TIOCSPGRP       0x5410
#define LINUX_TIOCOUTQ        0x5411
#define LINUX_TIOCSTI         0x5412
#define LINUX_TIOCGWINSZ      0x5413
#define LINUX_TIOCSWINSZ      0x5414
#define LINUX_TIOCMGET        0x5415
#define LINUX_TIOCMBIS        0x5416
#define LINUX_TIOCMBIC        0x5417
#define LINUX_TIOCMSET        0x5418
#define LINUX_TIOCGSOFTCAR    0x5419
#define LINUX_TIOCSSOFTCAR    0x541A
#define LINUX_FIONREAD        0x541B
#define LINUX_TIOCLINUX       0x541C
#define LINUX_TIOCCONS        0x541D
#define LINUX_TIOCGSERIAL     0x541E
#define LINUX_TIOCSSERIAL     0x541F
#define LINUX_TIOCPKT         0x5420
#define LINUX_FIONBIO         0x5421
#define LINUX_TIOCNOTTY       0x5422
#define LINUX_TIOCSETD        0x5423
#define LINUX_TIOCGETD        0x5424
#define LINUX_TCSBRKP         0x5425
#define LINUX_TIOCTTYGSTRUCT  0x5426
#define LINUX_FIONCLEX        0x5450
#define LINUX_FIOCLEX         0x5451
#define LINUX_FIOASYNC        0x5452
#define LINUX_TIOCSERCONFIG   0x5453
#define LINUX_TIOCSERGWILD    0x5454
#define LINUX_TIOCSERSWILD    0x5455
#define LINUX_TIOCGLCKTRMIOS  0x5456
#define LINUX_TIOCSLCKTRMIOS  0x5457
#define LINUX_TIOCSERGSTRUCT  0x5458
#define LINUX_TIOCSERGETLSR   0x5459
#define LINUX_TIOCSERGETMULTI 0x545A
#define LINUX_TIOCSERSETMULTI 0x545B

#include "translate.h"

void interrupt(Bit8u number, iInst_t inst, cpu_t *state);

#endif /* SYSCALL_H */
